@@ -11,7 +11,7 @@ The most commonly used functions/objects are:
 - wandb.save — save files in association with your run, like model weights
 - wandb.restore — restore the state of your code when you ran a given run
 
-For examples usage, see https://docs.wandb.com/library/example-projects
+For examples usage, see github.com/wandb/examples
 """
 
 from __future__ import absolute_import
@@ -19,10 +19,10 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = '0.10.7'
+__version__ = "0.10.11"
 
 # Used with pypi checks and other messages related to pip
-_wandb_module = 'wandb'
+_wandb_module = "wandb"
 
 import sys
 
@@ -39,23 +39,29 @@ else:
     TYPE_CHECKING = False
     from wandb import sdk_py27 as wandb_sdk
 
+import wandb
+
+wandb.wandb_lib = wandb_sdk.lib
+
 init = wandb_sdk.init
 setup = wandb_sdk.setup
 save = wandb_sdk.save
 watch = wandb_sdk.watch
+unwatch = wandb_sdk.unwatch
 finish = wandb_sdk.finish
 join = finish
 login = wandb_sdk.login
 helper = wandb_sdk.helper
 Artifact = wandb_sdk.Artifact
+AlertLevel = wandb_sdk.AlertLevel
 Settings = wandb_sdk.Settings
 Config = wandb_sdk.Config
 
 from wandb.apis import InternalApi, PublicApi
 from wandb.errors.error import CommError, UsageError
 
-from wandb.lib import preinit as _preinit
-from wandb.lib import lazyloader as _lazyloader
+_preinit = wandb_lib.preinit
+_lazyloader = wandb_lib.lazyloader
 from wandb import wandb_torch
 from wandb import util
 
@@ -70,6 +76,8 @@ from wandb.data_types import Html
 from wandb.data_types import Object3D
 from wandb.data_types import Molecule
 from wandb.data_types import Histogram
+from wandb.data_types import Classes
+from wandb.data_types import JoinedTable
 
 from wandb.wandb_agent import agent
 from wandb.wandb_controller import sweep, controller
@@ -96,8 +104,6 @@ def _is_internal_process():
     return _IS_INTERNAL_PROCESS
 
 
-from wandb.lib.ipython import _get_python_type
-
 # toplevel:
 # save()
 # restore()
@@ -109,17 +115,15 @@ from wandb.lib.ipython import _get_python_type
 Api = PublicApi
 api = InternalApi()
 run = None
-config = _preinit.PreInitObject("wandb.config")
-summary = _preinit.PreInitObject("wandb.summary")
-log = _preinit.PreInitCallable(
-    "wandb.log", wandb_sdk.wandb_run.Run.log
+config = _preinit.PreInitCallable(
+    _preinit.PreInitObject("wandb.config"), wandb_sdk.wandb_config.Config
 )
-save = _preinit.PreInitCallable(
-    "wandb.save", wandb_sdk.wandb_run.Run.save
-)
-restore = _preinit.PreInitCallable(
-    "wandb.restore", wandb_sdk.wandb_run.Run.restore
+summary = _preinit.PreInitCallable(
+    _preinit.PreInitObject("wandb.summary"), wandb_sdk.wandb_summary.Summary
 )
+log = _preinit.PreInitCallable("wandb.log", wandb_sdk.wandb_run.Run.log)
+save = _preinit.PreInitCallable("wandb.save", wandb_sdk.wandb_run.Run.save)
+restore = wandb_sdk.wandb_run.restore
 use_artifact = _preinit.PreInitCallable(
     "wandb.use_artifact", wandb_sdk.wandb_run.Run.use_artifact
 )
@@ -129,15 +133,22 @@ log_artifact = _preinit.PreInitCallable(
 plot_table = _preinit.PreInitCallable(
     "wandb.plot_table", wandb_sdk.wandb_run.Run.plot_table
 )
+alert = _preinit.PreInitCallable("wandb.alert", wandb_sdk.wandb_run.Run.alert)
 
 # record of patched libraries
 patched = {"tensorboard": [], "keras": [], "gym": []}
 
 keras = _lazyloader.LazyLoader("wandb.keras", globals(), "wandb.integration.keras")
 sklearn = _lazyloader.LazyLoader("wandb.sklearn", globals(), "wandb.sklearn")
-tensorflow = _lazyloader.LazyLoader("wandb.tensorflow", globals(), "wandb.integration.tensorflow")
-xgboost = _lazyloader.LazyLoader("wandb.xgboost", globals(), "wandb.integration.xgboost")
-tensorboard = _lazyloader.LazyLoader("wandb.tensorboard", globals(), "wandb.integration.tensorboard")
+tensorflow = _lazyloader.LazyLoader(
+    "wandb.tensorflow", globals(), "wandb.integration.tensorflow"
+)
+xgboost = _lazyloader.LazyLoader(
+    "wandb.xgboost", globals(), "wandb.integration.xgboost"
+)
+tensorboard = _lazyloader.LazyLoader(
+    "wandb.tensorboard", globals(), "wandb.integration.tensorboard"
+)
 gym = _lazyloader.LazyLoader("wandb.gym", globals(), "wandb.integration.gym")
 lightgbm = _lazyloader.LazyLoader(
     "wandb.lightgbm", globals(), "wandb.integration.lightgbm"
@@ -146,6 +157,7 @@ docker = _lazyloader.LazyLoader("wandb.docker", globals(), "wandb.docker")
 jupyter = _lazyloader.LazyLoader("wandb.jupyter", globals(), "wandb.jupyter")
 sacred = _lazyloader.LazyLoader("wandb.sacred", globals(), "wandb.integration.sacred")
 
+
 def ensure_configured():
     global api
     api = InternalApi()
