@@ -6,6 +6,7 @@ import os
 import platform
 import re
 import shutil
+import sys
 import tempfile
 import time
 
@@ -19,12 +20,20 @@ import wandb
 from wandb import __version__, env, util
 from wandb.apis.internal import Api as InternalApi
 from wandb.apis.normalize import normalize_exceptions
+from wandb.data_types import WBValue
 from wandb.errors.term import termlog
-from wandb.interface import artifacts
 from wandb.old.retry import retriable
 from wandb.old.summary import HTTPSummary
 import yaml
 
+
+# TODO: consolidate dynamic imports
+PY3 = sys.version_info.major == 3 and sys.version_info.minor >= 6
+if PY3:
+    from wandb.sdk.interface import artifacts
+else:
+    from wandb.sdk_py27.interface import artifacts
+
 logger = logging.getLogger(__name__)
 
 # Only retry requests for 20 seconds in the public api
@@ -156,6 +165,10 @@ class RetryingClient(object):
     def __init__(self, client):
         self._client = client
 
+    @property
+    def app_url(self):
+        return util.app_url(self._client.transport.url).replace("/graphql", "/")
+
     @retriable(
         retry_timedelta=RETRY_TIMEDELTA,
         check_retry_fn=util.no_retry_auth,
@@ -175,7 +188,7 @@ class Api(object):
             wandb.Api()
         ```
 
-    Args:
+    Arguments:
         overrides (dict): You can set `base_url` if you are using a wandb server
             other than https://api.wandb.ai.
             You can also set defaults for `entity`, `project`, and `run`.
@@ -324,14 +337,14 @@ class Api(object):
 
     def projects(self, entity=None, per_page=200):
         """Get projects for a given entity.
-        Args:
+        Arguments:
             entity (str): Name of the entity requested.  If None will fallback to
-                default entity passed to :obj:`Api`.  If no default entity, will raise a `ValueError`.
+                default entity passed to `Api`.  If no default entity, will raise a `ValueError`.
             per_page (int): Sets the page size for query pagination.  None will use the default size.
                 Usually there is no reason to change this.
 
         Returns:
-            A :obj:`Projects` object which is an iterable collection of :obj:`Project` objects.
+            A `Projects` object which is an iterable collection of `Project` objects.
 
         """
         if entity is None:
@@ -349,14 +362,14 @@ class Api(object):
 
         WARNING: This api is in beta and will likely change in a future release
 
-        Args:
+        Arguments:
             path (str): path to project the report resides in, should be in the form: "entity/project"
             name (str): optional name of the report requested.
             per_page (int): Sets the page size for query pagination.  None will use the default size.
                 Usually there is no reason to change this.
 
         Returns:
-            A :obj:`Reports` object which is an iterable collection of :obj:`BetaReport` objects.
+            A `Reports` object which is an iterable collection of `BetaReport` objects.
         """
         entity, project, run = self._parse_path(path)
         if entity is None:
@@ -399,7 +412,7 @@ class Api(object):
             ```
 
 
-        Args:
+        Arguments:
             path (str): path to project, should be in the form: "entity/project"
             filters (dict): queries for specific runs using the MongoDB query language.
                 You can filter by run properties such as config.key, summary_metrics.key, state, entity, createdAt, etc.
@@ -413,7 +426,7 @@ class Api(object):
                 The default order is run.created_at from newest to oldest.
 
         Returns:
-            A :obj:`Runs` object, which is an iterable collection of :obj:`Run` objects.
+            A `Runs` object, which is an iterable collection of `Run` objects.
         """
         entity, project = self._parse_project_path(path)
         key = path + str(filters) + str(order)
@@ -432,13 +445,13 @@ class Api(object):
     def run(self, path=""):
         """Returns a single run by parsing path in the form entity/project/run_id.
 
-        Args:
+        Arguments:
             path (str): path to run in the form entity/project/run_id.
                 If api.entity is set, this can be in the form project/run_id
                 and if api.project is set this can just be the run_id.
 
         Returns:
-            A :obj:`Run` object.
+            A `Run` object.
         """
         entity, project, run = self._parse_path(path)
         if not self._runs.get(path):
@@ -450,13 +463,13 @@ class Api(object):
         """
         Returns a sweep by parsing path in the form entity/project/sweep_id.
 
-        Args:
+        Arguments:
             path (str, optional): path to sweep in the form entity/project/sweep_id.  If api.entity
                 is set, this can be in the form project/sweep_id and if api.project is set
                 this can just be the sweep_id.
 
         Returns:
-            A :obj:`Sweep` object.
+            A `Sweep` object.
         """
         entity, project, sweep_id = self._parse_path(path)
         if not self._sweeps.get(path):
@@ -483,7 +496,7 @@ class Api(object):
     def artifact(self, name, type=None):
         """Returns a single artifact by parsing path in the form entity/project/run_id.
 
-        Args:
+        Arguments:
             name (str): An artifact name. May be prefixed with entity/project. Valid names
                 can be in the following forms:
                     name:version
@@ -491,7 +504,7 @@ class Api(object):
                     digest
             type (str, optional): The type of artifact to fetch.
         Returns:
-            A :obj:`Artifact` object.
+            A `Artifact` object.
         """
         if name is None:
             raise ValueError("You must specify name= to fetch an artifact.")
@@ -602,7 +615,7 @@ class User(Attrs):
 
 class Projects(Paginator):
     """
-    An iterable collection of :obj:`Project` objects.
+    An iterable collection of `Project` objects.
     """
 
     QUERY = gql(
@@ -685,7 +698,7 @@ class Project(Attrs):
 
 class Runs(Paginator):
     """An iterable collection of runs associated with a project and optional filter.
-    This is generally used indirectly via the :obj:`Api`.runs method
+    This is generally used indirectly via the `Api`.runs method
     """
 
     QUERY = gql(
@@ -1056,37 +1069,37 @@ class Run(Attrs):
     @normalize_exceptions
     def files(self, names=[], per_page=50):
         """
-        Args:
+        Arguments:
             names (list): names of the requested files, if empty returns all files
             per_page (int): number of results per page
 
         Returns:
-            A :obj:`Files` object, which is an iterator over :obj:`File` obejcts.
+            A `Files` object, which is an iterator over `File` obejcts.
         """
         return Files(self.client, self, names, per_page)
 
     @normalize_exceptions
     def file(self, name):
         """
-        Args:
+        Arguments:
             name (str): name of requested file.
 
         Returns:
-            A :obj:`File` matching the name argument.
+            A `File` matching the name argument.
         """
         return Files(self.client, self, [name])[0]
 
     @normalize_exceptions
     def upload_file(self, path, root="."):
         """
-        Args:
+        Arguments:
             path (str): name of file to upload.
             root (str): the root path to save the file relative to.  i.e.
                 If you want to have the file saved in the run as "my_dir/file.txt"
                 and you're currently in "my_dir" you would set root to "../"
 
         Returns:
-            A :obj:`File` matching the name argument.
+            A `File` matching the name argument.
         """
         api = InternalApi(
             default_settings={"entity": self.entity, "project": self.project},
@@ -1107,7 +1120,7 @@ class Run(Attrs):
         Returns sampled history metrics for a run.  This is simpler and faster if you are ok with
         the history records being sampled.
 
-        Args:
+        Arguments:
             samples (int, optional): The number of samples to return
             pandas (bool, optional): Return a pandas dataframe
             keys (list, optional): Only return metrics for specific keys
@@ -1148,7 +1161,7 @@ class Run(Attrs):
             ```
 
 
-        Args:
+        Arguments:
             keys ([str], optional): only fetch these keys, and only fetch rows that have all of keys defined.
             page_size (int, optional): size of pages to fetch from the api
 
@@ -1194,11 +1207,11 @@ class Run(Attrs):
     def use_artifact(self, artifact):
         """ Declare an artifact as an input to a run.
 
-        Args:
-            artifact (:obj:`Artifact`): An artifact returned from
+        Arguments:
+            artifact (`Artifact`): An artifact returned from
                 `wandb.Api().artifact(name)`
         Returns:
-            A :obj:`Artifact` object.
+            A `Artifact` object.
         """
         api = InternalApi(
             default_settings={"entity": self.entity, "project": self.project},
@@ -1221,12 +1234,12 @@ class Run(Attrs):
     def log_artifact(self, artifact, aliases=None):
         """ Declare an artifact as output of a run.
 
-        Args:
-            artifact (:obj:`Artifact`): An artifact returned from
+        Arguments:
+            artifact (`Artifact`): An artifact returned from
                 `wandb.Api().artifact(name)`
             aliases (list, optional): Aliases to apply to this artifact
         Returns:
-            A :obj:`Artifact` object.
+            A `Artifact` object.
         """
         api = InternalApi(
             default_settings={"entity": self.entity, "project": self.project},
@@ -1270,7 +1283,7 @@ class Run(Attrs):
     def url(self):
         path = self.path
         path.insert(2, "runs")
-        return "https://app.wandb.ai/" + "/".join(path)
+        return self.client.app_url + "/".join(path)
 
     @property
     def lastHistoryStep(self):  # noqa: N802
@@ -1304,7 +1317,7 @@ class Sweep(Attrs):
       api.sweep(sweep_path)
 
     Attributes:
-        runs (:obj:`Runs`): list of runs
+        runs (`Runs`): list of runs
         id (str): sweep id
         project (str): name of project
         config (str): dictionary of sweep configuration
@@ -1417,6 +1430,12 @@ class Sweep(Attrs):
             urllib.parse.quote_plus(str(self.id)),
         ]
 
+    @property
+    def url(self):
+        path = self.path
+        path.insert(2, "sweeps")
+        return self.client.app_url + "/".join(path)
+
     @classmethod
     def get(
         cls,
@@ -1474,7 +1493,7 @@ class Sweep(Attrs):
 
 
 class Files(Paginator):
-    """Files is an iterable collection of :obj:`File` objects."""
+    """Files is an iterable collection of `File` objects."""
 
     QUERY = gql(
         """
@@ -1599,7 +1618,7 @@ class File(object):
     def download(self, root=".", replace=False):
         """Downloads a file previously saved by a run from the wandb server.
 
-        Args:
+        Arguments:
             replace (boolean): If `True`, download will overwrite a local file
                 if it exists. Defaults to `False`.
             root (str): Local directory to save the file.  Defaults to ".".
@@ -1620,7 +1639,7 @@ class File(object):
 
 
 class Reports(Paginator):
-    """Reports is an iterable collection of :obj:`BetaReport` objects."""
+    """Reports is an iterable collection of `BetaReport` objects."""
 
     QUERY = gql(
         """
@@ -1804,7 +1823,7 @@ class BetaReport(Attrs):
     Attributes:
         name (string): report name
         description (string): report descirpiton;
-        user (:obj:User): the user that created the report
+        user (User): the user that created the report
         spec (dict): the spec off the report;
         updated_at (string): timestamp of last update
     """
@@ -2360,6 +2379,69 @@ class ArtifactCollection(object):
 
 
 class Artifact(object):
+    QUERY = gql(
+        """
+        query Artifact(
+            $id: ID!,
+        ) {
+            artifact(id: $id) {
+                currentManifest {
+                    id
+                    file {
+                        id
+                        directUrl
+                    }
+                }
+                ...ArtifactFragment
+            }
+        }
+        %s
+    """
+        % ARTIFACT_FRAGMENT
+    )
+
+    @classmethod
+    def from_id(cls, artifact_id, client):
+        artifact = artifacts.get_artifacts_cache().get_artifact(artifact_id)
+        if artifact is not None:
+            return artifact
+        response = client.execute(Artifact.QUERY, variable_values={"id": id},)
+
+        name = None
+        if response.get("artifact") is not None:
+            if response["artifact"].get("aliases") is not None:
+                aliases = response["artifact"]["aliases"]
+                name = ":".join(
+                    [aliases[0]["artifactCollectionName"], aliases[0]["alias"]]
+                )
+                if len(aliases) > 1:
+                    for alias in aliases:
+                        if alias["alias"] != "latest":
+                            name = ":".join(
+                                [alias["artifactCollectionName"], alias["alias"]]
+                            )
+                            break
+
+            artifact = cls(
+                client=client,
+                entity=None,
+                project=None,
+                name=name,
+                attrs=response["artifact"],
+            )
+            index_file_url = response["artifact"]["currentManifest"]["file"][
+                "directUrl"
+            ]
+            with requests.get(index_file_url) as req:
+                req.raise_for_status()
+                artifact._manifest = artifacts.ArtifactManifest.from_manifest_json(
+                    artifact, json.loads(req.content)
+                )
+
+            artifact._load_dependent_manifests()
+
+            return artifact
+
     def __init__(self, client, entity, project, name, attrs=None):
         self.client = client
         self.entity = entity
@@ -2380,6 +2462,7 @@ class Artifact(object):
         ]
         self._manifest = None
         self._is_downloaded = False
+        artifacts.get_artifacts_cache().store_artifact(self)
 
     @property
     def id(self):
@@ -2477,14 +2560,20 @@ class Artifact(object):
         raise ValueError("Cannot add files to an artifact once it has been saved")
 
     def get_path(self, name):
-        manifest = self._load_manifest()
+        parent_self = self
+        manifest = parent_self._load_manifest()
         storage_policy = manifest.storage_policy
+        default_root = parent_self._default_root()
 
         entry = manifest.entries.get(name)
         if entry is None:
             raise KeyError("Path not contained in artifact: %s" % name)
 
         class ArtifactEntry(object):
+            def __init__(self):
+                self.parent_artifact = parent_self
+                self.name = name
+
             @staticmethod
             def copy(cache_path, target_path):
                 # can't have colons in Windows
@@ -2505,52 +2594,88 @@ class Artifact(object):
 
             @staticmethod
             def download(root=None):
+                root = root or default_root
                 if entry.ref is not None:
-                    return storage_policy.load_reference(
-                        self, name, manifest.entries[name], local=True
+                    cache_path = storage_policy.load_reference(
+                        parent_self, name, manifest.entries[name], local=True
+                    )
+                else:
+                    cache_path = storage_policy.load_file(
+                        parent_self, name, manifest.entries[name]
                     )
 
-                cache_path = storage_policy.load_file(
-                    self, name, manifest.entries[name]
-                )
-                if root is not None:
-                    return ArtifactEntry().copy(cache_path, os.path.join(root, name))
-                return cache_path
+                return ArtifactEntry().copy(cache_path, os.path.join(root, name))
 
             @staticmethod
             def ref():
                 if entry.ref is not None:
                     return storage_policy.load_reference(
-                        self, name, manifest.entries[name], local=False
+                        parent_self, name, manifest.entries[name], local=False
                     )
                 raise ValueError("Only reference entries support ref().")
 
+            @staticmethod
+            def ref_url():
+                return (
+                    "wandb-artifact://"
+                    + util.b64_to_hex_id(parent_self.id)
+                    + "/"
+                    + name
+                )
+
         return ArtifactEntry()
 
+    def get(self, name):
+        """Returns the wandb.Media resource stored in the artifact. Media can be
+        stored in the artifact via Artifact#add(obj: wandbMedia, name: str)`
+        Arguments:
+            name (str): name of resource.
+
+        Returns:
+            A `wandb.Media` which has been stored at `name`
+        """
+        self._load_manifest()
+
+        type_mapping = WBValue.type_mapping()
+        for artifact_type_str in type_mapping:
+            wb_class = type_mapping[artifact_type_str]
+            wandb_file_name = wb_class.with_suffix(name)
+            entry = self._manifest.entries.get(wandb_file_name)
+            if entry is not None:
+                # If the entry is a reference from another artifact, then get it directly from that artifact
+                if self._manifest_entry_is_artifact_reference(entry):
+                    artifact = self._get_ref_artifact_from_entry(entry)
+                    return artifact.get(util.uri_from_path(entry.ref))
+
+                # Get the ArtifactEntry
+                item = self.get_path(wandb_file_name)
+                item_path = item.download()
+
+                # Load the object from the JSON blob
+                result = None
+                json_obj = {}
+                with open(item_path, "r") as file:
+                    json_obj = json.load(file)
+                result = wb_class.from_json(json_obj, self)
+                result.artifact_source = {"artifact": self, "name": name}
+                return result
+
     def download(self, root=None):
         """Download the artifact to dir specified by the <root>
 
-        Args:
+        Arguments:
             root (str, optional): directory to download artifact to. If None
                 artifact will be downloaded to './artifacts/<self.name>/'
 
         Returns:
             The path to the downloaded contents.
         """
-        dirpath = root
-        if dirpath is None:
-            dirpath = os.path.join(".", "artifacts", self.name)
-            if platform.system() == "Windows":
-                head, tail = os.path.splitdrive(dirpath)
-                dirpath = head + tail.replace(":", "-")
-
+        dirpath = root or self._default_root()
         manifest = self._load_manifest()
         nfiles = len(manifest.entries)
         size = sum(e.size for e in manifest.entries.values())
         log = False
         if nfiles > 5000 or size > 50 * 1024 * 1024:
-            log = True
-        if log:
             termlog(
                 "Downloading large artifact %s, %.2fMB. %s files... "
                 % (self.artifact_name, size / (1024 * 1024), nfiles),
@@ -2563,7 +2688,7 @@ class Artifact(object):
         import multiprocessing.dummy  # this uses threads
 
         pool = multiprocessing.dummy.Pool(32)
-        pool.map(partial(self._download_file, dirpath=dirpath), manifest.entries)
+        pool.map(partial(self._download_file, root=dirpath), manifest.entries)
         pool.close()
         pool.join()
 
@@ -2571,12 +2696,13 @@ class Artifact(object):
 
         if log:
             termlog("Done. %.1fs" % (time.time() - start_time), prefix=False)
+
         return dirpath
 
     def file(self, root=None):
         """Download a single file artifact to dir specified by the <root>
 
-        Args:
+        Arguments:
             root (str, optional): directory to download artifact to. If None
                 artifact will be downloaded to './artifacts/<self.name>/'
 
@@ -2594,11 +2720,18 @@ class Artifact(object):
                 'This artifact contains more than one file, call `.download()` to get all files or call .get_path("filename").download()'
             )
 
-        return self._download_file(list(manifest.entries)[0], root)
+        return self._download_file(list(manifest.entries)[0], root=root)
 
-    def _download_file(self, name, dirpath):
+    def _download_file(self, name, root):
         # download file into cache and copy to target dir
-        return self.get_path(name).download(dirpath)
+        return self.get_path(name).download(root)
+
+    def _default_root(self):
+        root = os.path.join(".", "artifacts", self.name)
+        if platform.system() == "Windows":
+            head, tail = os.path.splitdrive(root)
+            root = head + tail.replace(":", "-")
+        return root
 
     @normalize_exceptions
     def save(self):
@@ -2646,7 +2779,7 @@ class Artifact(object):
         Raises a ValueError if the verification fails. Does not verify downloaded
         reference files.
 
-        Args:
+        Arguments:
             root (str, optional): directory to download artifact to. If None
                 artifact will be downloaded to './artifacts/<self.name>/'
         """
@@ -2766,12 +2899,37 @@ class Artifact(object):
                 self._manifest = artifacts.ArtifactManifest.from_manifest_json(
                     self, json.loads(req.content)
                 )
+
+            self._load_dependent_manifests()
+
         return self._manifest
 
+    def _load_dependent_manifests(self):
+        """Helper function to interrogate entries and ensure we have loaded their manifests"""
+        # Make sure dependencies are avail
+        for entry_key in self._manifest.entries:
+            entry = self._manifest.entries[entry_key]
+            if self._manifest_entry_is_artifact_reference(entry):
+                dep_artifact = self._get_ref_artifact_from_entry(entry)
+                dep_artifact._load_manifest()
+
+    @staticmethod
+    def _manifest_entry_is_artifact_reference(entry):
+        """Helper function determines if an ArtifactEntry in manifest is an artifact reference"""
+        return (
+            entry.ref is not None
+            and urllib.parse.urlparse(entry.ref).scheme == "wandb-artifact"
+        )
+
+    def _get_ref_artifact_from_entry(self, entry):
+        """Helper function returns the referenced artifact from an entry"""
+        artifact_id = util.host_from_path(entry.ref)
+        return Artifact.from_id(util.hex_to_b64_id(artifact_id), self.client)
+
 
 class ArtifactVersions(Paginator):
     """An iterable collection of artifact versions associated with a project and optional filter.
-    This is generally used indirectly via the :obj:`Api`.artifact_versions method
+    This is generally used indirectly via the `Api`.artifact_versions method
     """
 
     QUERY = gql(
