@@ -187,7 +187,7 @@ class WandbCallback(keras.callbacks.Callback):
 
     WandbCallback can optionally save training and validation data for wandb to visualize.
 
-    Args:
+    Arguments:
         monitor (str): name of metric to monitor.  Defaults to val_loss.
         mode (str): one of {"auto", "min", "max"}.
             "min" - save model when monitor is minimized
@@ -744,6 +744,8 @@ class WandbCallback(keras.callbacks.Callback):
             return None
 
     def _save_model(self, epoch):
+        if wandb.run.disabled:
+            return
         if self.verbose > 0:
             print(
                 "Epoch %05d: %s improved from %0.5f to %0.5f,"
