@@ -3,7 +3,8 @@ import abc
 
 import six
 import wandb
-from wandb.interface.summary_record import SummaryItem, SummaryRecord
+
+from .interface.summary_record import SummaryItem, SummaryRecord
 
 if wandb.TYPE_CHECKING:  # type: ignore
     import typing as t
@@ -79,7 +80,37 @@ class SummaryDict(object):
 
 
 class Summary(SummaryDict):
-    """Root node of the summary data structure. Contains the callback."""
+    """
+    Summary tracks single values for each run. By default, summary is set to the
+    last value of History.
+
+    For example, wandb.log({'accuracy': 0.9}) will add a new step to History and
+    update Summary to the latest value. In some cases, it's more useful to have
+    the maximum or minimum of a metric instead of the final value. You can set
+    history manually (wandb.summary['accuracy'] = best_acc).
+
+    In the UI, summary metrics appear in the table to compare across runs.
+    Summary metrics are also used in visualizations like the scatter plot and
+    parallel coordinates chart.
+
+    After training has completed, you may want to save evaluation metrics to a
+    run. Summary can handle numpy arrays and PyTorch/TensorFlow tensors. When
+    you save one of these types to Summary, we persist the entire tensor in a
+    binary file and store high level metrics in the summary object, such as min,
+    mean, variance, and 95th percentile.
+
+    Examples:
+        ```
+        wandb.init(config=args)
+
+        best_accuracy = 0
+        for epoch in range(1, args.epochs + 1):
+        test_loss, test_accuracy = test()
+        if (test_accuracy > best_accuracy):
+            wandb.run.summary["best_accuracy"] = test_accuracy
+            best_accuracy = test_accuracy
+        ```
+    """
 
     _update_callback: t.Callable
     _get_current_summary_callback: t.Callable
